@@ -263,8 +263,8 @@ generated by running puppet agent with '--genconfig'.
 
 * --test:
   Enable the most common options used for testing. These are 'onetime',
-  'verbose', 'ignorecache', 'no-daemonize', 'no-usecacheonfailure',
-  'detailed-exitcodes', 'no-splay', and 'show_diff'.
+  'verbose', 'no-daemonize', 'no-usecacheonfailure', 'detailed-exitcodes',
+  'no-splay', and 'show_diff'.
 
 * --verbose:
   Turn on verbose reporting.
@@ -373,7 +373,6 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
 
   # Enable all of the most common test options.
   def setup_test
-    Puppet.settings.handlearg("--ignorecache")
     Puppet.settings.handlearg("--no-usecacheonfailure")
     Puppet.settings.handlearg("--no-splay")
     Puppet.settings.handlearg("--show_diff")
