@@ -267,6 +267,8 @@ class Puppet::Resource::Catalog::Compiler < Puppet::Indirector::Code
         rescue Puppet::Error => detail
           Puppet.err(detail.to_s) if networked?
           raise
+        ensure
+          Puppet::Type.clear_misses unless Puppet[:always_retry_plugins]
         end
 
         if checksum_type && config.is_a?(model)
