@@ -154,6 +154,12 @@ class AccessOperator
     Types::TypeFactory.variant(*keys)
   end
 
+  def access_PSemVerType(o, scope, keys)
+    keys.flatten!
+    assert_keys(keys, o, 1, Float::INFINITY, String, Semantic::VersionRange)
+    Types::TypeFactory.sem_ver(*keys)
+  end
+
   def access_PTupleType(o, scope, keys)
     keys.flatten!
     if Types::TypeFactory.is_range_parameter?(keys[-2]) && Types::TypeFactory.is_range_parameter?(keys[-1])
@@ -260,6 +266,15 @@ class AccessOperator
     end
   end
 
+  def access_PObjectType(o, scope, keys)
+    keys.flatten!
+    if keys.size == 1
+      Types::TypeFactory.object(keys[0])
+    else
+      fail(Issues::BAD_TYPE_SLICE_ARITY, @semantic, {:base_type => 'Object-Type', :min => 1, :actual => keys.size})
+    end
+  end
+
   def access_PNotUndefType(o, scope, keys)
     keys.flatten!
     case keys.size
@@ -357,6 +372,10 @@ class AccessOperator
   #
   def access_PHashType(o, scope, keys)
     keys.flatten!
+    if keys.size == 2 && keys[0].is_a?(Integer) && keys[1].is_a?(Integer)
+      return Types::PHashType.new(nil, nil, Types::PIntegerType.new(*keys))
+    end
+
     keys[0,2].each_with_index do |k, index|
       unless k.is_a?(Types::PAnyType)
         fail(Issues::BAD_TYPE_SLICE_TYPE, @semantic.keys[index], {:base_type => 'Hash-Type', :actual => k.class})
@@ -364,18 +383,18 @@ class AccessOperator
     end
     case keys.size
     when 2
-      Types::PHashType.new(keys[0], keys[1], nil)
+      size_t = nil
     when 3
       size_t = keys[2]
       size_t = Types::PIntegerType.new(size_t) unless size_t.is_a?(Types::PIntegerType)
-      Types::PHashType.new(keys[0], keys[1], size_t)
     when 4
-      Types::PHashType.new(keys[0], keys[1], collection_size_t(1, keys[2], keys[3]))
+      size_t = collection_size_t(2, keys[2], keys[3])
     else
       fail(Issues::BAD_TYPE_SLICE_ARITY, @semantic, {
         :base_type => 'Hash-Type', :min => 2, :max => 4, :actual => keys.size
       })
     end
+    Types::PHashType.new(keys[0], keys[1], size_t)
   end
 
   # CollectionType is parameterized with a range
@@ -383,9 +402,9 @@ class AccessOperator
     keys.flatten!
     case keys.size
     when 1
-      size_t = collection_size_t(1, keys[0])
+      size_t = collection_size_t(0, keys[0])
     when 2
-      size_t = collection_size_t(1, keys[0], keys[1])
+      size_t = collection_size_t(0, keys[0], keys[1])
     else
       fail(Issues::BAD_TYPE_SLICE_ARITY, @semantic,
         {:base_type => 'Collection-Type', :min => 1, :max => 2, :actual => keys.size})
@@ -399,19 +418,31 @@ class AccessOperator
     keys.flatten!
     case keys.size
     when 1
+      unless keys[0].is_a?(Types::PAnyType)
+        fail(Issues::BAD_TYPE_SLICE_TYPE, @semantic.keys[0], {:base_type => 'Array-Type', :actual => keys[0].class})
+      end
+      type = keys[0]
       size_t = nil
     when 2
-      size_t = collection_size_t(1, keys[1])
+      if keys[0].is_a?(Types::PAnyType)
+        size_t = collection_size_t(1, keys[1])
+        type = keys[0]
+      else
+        size_t = collection_size_t(0, keys[0], keys[1])
+        type = nil
+      end
     when 3
-      size_t = collection_size_t(1, keys[1], keys[2])
+      if keys[0].is_a?(Types::PAnyType)
+        size_t = collection_size_t(1, keys[1], keys[2])
+        type = keys[0]
+      else
+        fail(Issues::BAD_TYPE_SLICE_TYPE, @semantic.keys[0], {:base_type => 'Array-Type', :actual => keys[0].class})
+      end
     else
       fail(Issues::BAD_TYPE_SLICE_ARITY, @semantic,
         {:base_type => 'Array-Type', :min => 1, :max => 3, :actual => keys.size})
     end
-    unless keys[0].is_a?(Types::PAnyType)
-      fail(Issues::BAD_TYPE_SLICE_TYPE, @semantic.keys[0], {:base_type => 'Array-Type', :actual => keys[0].class})
-    end
-    Types::PArrayType.new(keys[0], size_t)
+    Types::PArrayType.new(type, size_t)
   end
 
   # Produces an PIntegerType (range) given one or two keys.
@@ -438,6 +469,11 @@ class AccessOperator
     access(t, scope, *keys)
   end
 
+  # If a type reference is encountered here, it's an error
+  def access_PTypeReferenceType(o, scope, keys)
+    fail(Issues::UNKNOWN_RESOURCE_TYPE, @semantic, {:type_name => o.type_string })
+  end
+
   # A Resource can create a new more specific Resource type, and/or an array of resource types
   # If the given type has title set, it can not be specified further.
   # @example
@@ -461,12 +497,12 @@ class AccessOperator
     # Must know which concrete resource type to operate on in all cases.
     # It is not allowed to specify the type in an array arg - e.g. Resource[[File, 'foo']]
     # type_name is LHS type_name if set, else the first given arg
-    type_name = o.type_name || keys.shift
+    type_name = o.type_name || Types::TypeFormatter.singleton.capitalize_segments(keys.shift)
     type_name = case type_name
     when Types::PResourceType
       type_name.type_name
     when String
-      type_name.downcase
+      type_name
     else
       # blame given left expression if it defined the type, else the first given key expression
       blame = o.type_name.nil? ? @semantic.keys[0] : @semantic.left_expr
@@ -474,7 +510,7 @@ class AccessOperator
     end
 
     # type name must conform
-    if type_name !~ Patterns::CLASSREF
+    if type_name !~ Patterns::CLASSREF_EXT
       fail(Issues::ILLEGAL_CLASSREF, blamed, {:name=>type_name})
     end
 
@@ -565,6 +601,8 @@ class AccessOperator
                  c.type_name
                elsif c.is_a?(String)
                  c.downcase
+               elsif c.is_a?(Types::PTypeReferenceType)
+                 c.type_string.downcase
                else
                  fail(Issues::ILLEGAL_HOSTCLASS_NAME, @semantic.keys[i], {:name => c})
                end
