@@ -1,7 +1,8 @@
 # This class is the backing implementation of the Puppet function 'lookup'.
 # See puppet/functions/lookup.rb for documentation.
 #
-module Puppet::Pops::Lookup
+module Puppet::Pops
+module Lookup
   LOOKUP_OPTIONS = 'lookup_options'.freeze
   GLOBAL = '__global__'.freeze
 
@@ -11,25 +12,25 @@ module Puppet::Pops::Lookup
   # See puppet/functions/lookup.rb for full documentation and all parameter combinations.
   #
   # @param name [String|Array<String>] The name or names to lookup
-  # @param type [Puppet::Pops::Types::PAnyType|nil] The expected type of the found value
+  # @param type [Types::PAnyType|nil] The expected type of the found value
   # @param default_value [Object] The value to use as default when no value is found
   # @param has_default [Boolean] Set to _true_ if _default_value_ is included (_nil_ is a valid _default_value_)
-  # @param merge [Puppet::Pops::MergeStrategy,String,Hash<String,Object>,nil] Merge strategy or hash with strategy and options
-  # @param lookup_invocation [Puppet::Pops::Lookup::Invocation] Invocation data containing scope, overrides, and defaults
+  # @param merge [MergeStrategy,String,Hash<String,Object>,nil] Merge strategy or hash with strategy and options
+  # @param lookup_invocation [Invocation] Invocation data containing scope, overrides, and defaults
   # @return [Object] The found value
   #
   def self.lookup(name, value_type, default_value, has_default, merge, lookup_invocation)
-    value_type = Puppet::Pops::Types::PDataType::DEFAULT if value_type.nil?
+    value_type = Types::PDataType::DEFAULT if value_type.nil?
     names = name.is_a?(Array) ? name : [name]
 
     # find first name that yields a non-nil result and wrap it in a two element array
     # with name and value.
-    not_found = Puppet::Pops::MergeStrategy::NOT_FOUND
+    not_found = MergeStrategy::NOT_FOUND
     override_values = lookup_invocation.override_values
     result_with_name = names.reduce([nil, not_found]) do |memo, key|
-      value = override_values.include?(key) ? assert_type('override', value_type, override_values[key]) : not_found
+      value = override_values.include?(key) ? assert_type(["Value found for key '%s' in override hash", key], value_type, override_values[key]) : not_found
       catch(:no_such_key) { value = search_and_merge(key, lookup_invocation, merge) } if value.equal?(not_found)
-      break [key, assert_type('found', value_type, value)] unless value.equal?(not_found)
+      break [key, assert_type('Found value', value_type, value)] unless value.equal?(not_found)
       memo
     end
 
@@ -38,7 +39,7 @@ module Puppet::Pops::Lookup
       default_values = lookup_invocation.default_values
       unless default_values.empty?
         result_with_name = names.reduce(result_with_name) do |memo, key|
-          value = default_values.include?(key) ? assert_type('default_values_hash', value_type, default_values[key]) : not_found
+          value = default_values.include?(key) ? assert_type(["Value found for key '%s' in default values hash", key], value_type, default_values[key]) : not_found
           memo = [key, value]
           break memo unless value.equal?(not_found)
           memo
@@ -49,9 +50,9 @@ module Puppet::Pops::Lookup
     answer = result_with_name[1]
     if answer.equal?(not_found)
       if block_given?
-        answer = assert_type('default_block', value_type, yield(name))
+        answer = assert_type('Value returned from default block', value_type, yield(name))
       elsif has_default
-        answer = assert_type('default_value', value_type, default_value)
+        answer = assert_type('Default value', value_type, default_value)
       else
         fail_lookup(names)
       end
@@ -65,7 +66,7 @@ module Puppet::Pops::Lookup
   end
 
   def self.assert_type(subject, type, value)
-    Puppet::Pops::Types::TypeAsserter.assert_instance_of(subject, type, value)
+    Types::TypeAsserter.assert_instance_of(subject, type, value)
   end
   private_class_method :assert_type
 
@@ -75,3 +76,4 @@ module Puppet::Pops::Lookup
   end
   private_class_method :fail_lookup
 end
+end
