@@ -28,7 +28,7 @@ Puppet::Type.type(:package).provide :dnf, :parent => :yum do
       end
   end
 
-  defaultfor :operatingsystem => :fedora, :operatingsystemmajrelease => ['22', '23']
+  defaultfor :operatingsystem => :fedora, :operatingsystemmajrelease => ['22', '23', '24']
 
   # The value to pass to DNF as its error output level.
   # DNF differs from Yum slightly with regards to error outputting.
